-------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
-------------------------------------------

local LrApplication = import 'LrApplication'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrShell = import 'LrShell'
local LrTasks = import 'LrTasks'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 

local myLogger = LrLogger('PhotomatixLogger')
myLogger:enable("print") 

--require 'ExportToDPXUtils'

ExportToDPXUploadTask = {}

function ExportToDPXUploadTask.outputToLog(message)
	myLogger:trace(message)
end

function ExportToDPXUploadTask.processRenderedPhotos( functionContext, exportContext )

	myLogger:trace('Start ExportToDPXUploadTask.processRenderedPhotos')

	local pmdebug = true
	
	
	
	local lrVersion=LrApplication.versionTable
	if lrVersion then
		prefs.LrVersion=lrVersion().major
	else
		prefs.LrVersion=1
	end

-------------------------- Showing plugin dialog and getting the options selected ------------------------------------	
	require 'ExportToDPXSettings'

	local exportSession = exportContext.exportSession
	if exportSession:countRenditions() ~= 1 then
		LrDialogs.message(LOC "$$$/Photomatix/General/Note=Note",
						  LOC "$$$/Photomatix/General/SinglePhoto=Please select a single photo to export", "info")
		return
	end

	ExportToDPXSettings.getOptions()
	
 	if prefs.processType=='none' then
		return
	end
	

-------------------------- Getting path to command line program (PhotomatixCL for WIN_ENV, MergeToHDR for MAC_ENV)  ------------------------------------	
	local appPath=LrPathUtils.child(_PLUGIN.path, "app")

	local clProgName="DPXConverter"

	appPath=LrPathUtils.child(appPath, clProgName)
	
	if (WIN_ENV) then
		appPath=LrPathUtils.addExtension(appPath, "exe")
	end

	if pmdebug then
		ExportToDPXUploadTask.outputToLog(string.format('appPath: %s', appPath))
	end

	if not LrFileUtils.exists(appPath) then
		ExportToDPXUploadTask.outputToLog(string.format('Could not find command line program at: %s',appPath))
		LrErrors.throwUserError(string.format("Could not find %s program in %s",clProgName, LrPathUtils.parent(appPath))) 		
		return
	end


-------------------------- Checking license key and showing registration dialog if necessary  ------------------------------------	

	local enteredKeyOK=false
	local hasEnteredKey=false
--[[
	local regOK=false
	if ((prefs.isRegistered~=nil) and (prefs.licenseKey~=nil)) then
		if (prefs.isRegistered) then
			regOK=true
		end
	end

	if not regOK then
--]]	

--[[
	local hasLicense=ExportToDPXUtils.Validate("")

	if not hasLicense then

		require 'ExportToDPXRegistration'
		local retRegDialog=ExportToDPXRegistration.showRegDialog()
			
		if ( (retRegDialog~=nil) and (retRegDialog=='cancel') ) then
			return
		end

		if ((prefs.enteredKey~=nil) and (prefs.enteredKey~="")) then
			hasEnteredKey=true
			enteredKeyOK=ExportToDPXUtils.Validate(prefs.enteredKey)
			if enteredKeyOK then
				prefs.isRegistered=true
				prefs.licenseKey=prefs.enteredKey
			else	
				prefs.enteredKey=""
			end
		end
	end
--	end
	
	if hasEnteredKey then
		if enteredKeyOK then
			dialogMessageOK=LOC "$$$/Photomatix/Registration/RegistrationMessageOK=The license key you entered is valid."		
			dialogInfoOK=LOC "$$$/Photomatix/Registration/RegistrationInfoOK=Thank you for registering your copy of the 'Merge to 32-bit HDR' plug-in. There won't be any watermark added to the merged 32-bit file."		
			LrDialogs.message(dialogMessageOK, dialogInfoOK)
		else
			dialogMessageFailed=LOC "$$$/Photomatix/Registration/RegistrationMessageFailed=Registration did not go through"		
			dialogInfoFailed=LOC "$$$/Photomatix/Registration/RegistrationInfoFailed=The license key you entered could not be validated. A watermark will be added to the merged 32-bit file."		
			local userActionConf = LrDialogs.confirm(dialogMessageFailed, dialogInfoFailed, LOC "$$$/Photomatix/General/Continue=Continue")
			if ( (userActionConf~=nil) and (userActionConf=='cancel') ) then
				return
			end
		end
	end
	
	--]]
	-- Temporary only
	-- *********************************
	------------------------------------
	prefs.licenseKey="temp"
	prefs.isRegistered=true
	------------------------------------

	
	local txtProgress=LOC "$$$/Photomatix/Progress/ExportPhotos=Exporting photo..."
	local nFiles=0	
	local retour=-1
	local tempPath = nil
	local photoToStackWith=nil
	local pathPhotoToStackWith=nil
	local outputTempFile
	local photosProcessed=false

	LrFunctionContext.callWithContext('ExecuteExportToDPX', function(context)

		local outerProgressScope = LrDialogs.showModalProgressDialog({
					title = LOC "$$$/Photomatix/General/ExportDPX=Export to DPX",
					caption = txtProgress,
					cannotCancel = true,
					functionContext = context,
				})

		outerProgressScope:setIndeterminate()

-------------------------- Setting path to export to in temporary directory  ------------------------------------	
	local sysTempPath = LrPathUtils.getStandardFilePath('temp')
	repeat
		local now = LrDate.currentTime()
		tempPath = LrPathUtils.child(sysTempPath, LrDate.timeToUserFormat(now, 'temp_lrexport2dpx_%m%d%H%M%S'))
		if LrFileUtils.exists(tempPath) then
			tempPath = nil
		else
			LrFileUtils.createAllDirectories(tempPath)
		end
	until tempPath
	
	if pmdebug then 
		ExportToDPXUploadTask.outputToLog(string.format('tempPath: %s',tempPath))
	end

	local exportSession = exportContext.exportSession
	
	local activeCatalog = LrApplication.activeCatalog()

	local exportParams = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()

	local progressScope = exportContext:configureProgress{
						title = LOC( "$$$/Photomatix/Upload/ProgressTransfer=Transferring photo for conversion to DPX"),
					}

	local files = {}
	local srcFiles = { }
	nFiles=0
	local inputFilesArgs=""
	
-------------------------- Exporting photos to temporary directory ------------------------------------

	for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
	
		local success, pathOrMessage = rendition:waitForRender()
		
		if progressScope:isCanceled() then break end
		
		if success then
			local copyPath = LrPathUtils.child(tempPath, LrPathUtils.leafName(pathOrMessage))
			LrFileUtils.copy(pathOrMessage, copyPath)
			table.insert(files, copyPath)
			table.insert(srcFiles, pathOrMessage)
			--inputFilesArgs=string.format('%s \"%s\"',inputFilesArgs,copyPath)
			nFiles = nFiles + 1
		else
			LrErrors.throwUserError("Lightroom rendition failed. Please check that the selected photos exist.")
			---	error(pathOrMessage)
		end 
	end

	--if nFiles > 1 then
		local doesExist=LrFileUtils.exists(appPath) 

		if doesExist then
		
-------------------------- Building command line arguments to pass to DPXConvert ------------------------------------
		
			
			ExportToDPXUploadTask.outputToLog('Start')
			ExportToDPXUploadTask.outputToLog(string.format('outDir=%s', prefs.finalOutDir))
			ExportToDPXUploadTask.outputToLog(string.format('srcFiles=%s', srcFiles[1]))
			ExportToDPXUploadTask.outputToLog(string.format('inFiles=%s', files[1]))
			
			local outDepth = prefs.outputBitDepth
			local outFile = LrPathUtils.leafName(srcFiles[1])
			outFile = LrPathUtils.removeExtension(outFile)
			outFile = LrPathUtils.addExtension(outFile, 'dpx')
			local outDir = prefs.finalOutDir
			outFile = LrPathUtils.child(prefs.finalOutDir, outFile)

			local PhotomatixClientCommand
			PhotomatixClientCommand=string.format('\"%s\" -bi %d %s %s', appPath, prefs.outputBitDepth, files[1], outFile)
				

			outputTempFile=string.format('%sCmdOutput', tempPath)
			if (WIN_ENV) then
				PhotomatixClientCommand=string.format('%s 2> \"%s\"', PhotomatixClientCommand, outputTempFile)		
				PhotomatixClientCommand=string.format('\"%s\"', PhotomatixClientCommand)
			else
				PhotomatixClientCommand=PhotomatixClientCommand  -- string.format('%s %s', PhotomatixClientCommand, prefs.processType, inputFilesArgs)		
			end

			progressScope:done()
				
			myLogger:trace(string.format('PhotomatixClientCommand: %s', PhotomatixClientCommand))

			if (WIN_ENV) then 
				LrFileUtils.delete(outputTempFile) -- in case it is there from before
			end
			
-------------------------- Calling command-line program ------------------------------------


			local txtProgress=LOC "$$$/Photomatix/Progress/Convert=Exporting to DPX..."
			
			outerProgressScope:setCaption(txtProgress)
			
--			LrFunctionContext.callWithContext('ExecuteCreateHDR', function(context)

--				local progressScope = LrDialogs.showModalProgressDialog({
--					title = LOC "$$$/Photomatix/General/ConvertToDPX=Convert to DPX",
--					caption = txtProgress,
--					cannotCancel = true,
--					functionContext = context,
--				})

				progressScope:setIndeterminate()

				retour=LrTasks.execute(PhotomatixClientCommand)
				photosProcessed = true
			
				outerProgressScope:done()

		end
		--end
	end)  -- End of section done while showing progress dialog
	
	if photosProcessed then
			
			myLogger:trace(string.format('retour PhotomatixClient: %d', retour))
			
			-- Remove temp files, because PhotomatixCL does not do this.
			if WIN_ENV then
				if string.find(tempPath, "lrexport2dpx") ~= nil then  -- Make extra-sure that this really is a temp path
					LrFileUtils.delete(tempPath)
				end
			end
-------------------------- Returning from command-line program ------------------------------------

			local processingSucceeded = true
			if WIN_ENV then
				local outputMessage
				if math.abs(retour) >= 0x40000000 and math.abs(retour) <= 0xff000000 and WIN_ENV then
					-- These seem to correspond to PhotomatiCL crashing on Windows, so we are not likely to get a sensible output message
					outputMessage = "Conversion failed"
					processingSucceeded = false
				elseif retour < 0 or retour > 0xff000000 then
					processingSucceeded = false
					-- PhotomatixCL finished, but returned an error code
					local handle = io.open(outputTempFile, 'rb')
					if handle then
						ExportToDPXUploadTask.outputToLog("Handle ok")
						while true do
							local currLine = handle:read('*line')
							
							if currLine == nil then break end
							outputMessage = currLine
						end
           				handle:close()
					end
				end
				
				ExportToDPXUploadTask.outputToLog(string.format('Succeeded: %s', processingSucceeded))
				if outputMessage == nil or string.find(outputMessage, "^%d+$") then  -- If just a number, there was no final message, and we just got one of the normal codes printed by PhotomatixCL
					outputMessage = ""
					if retour == 0xfffffffc then  -- -4: out of memory, and no custom error message was displayed
						outputMessage=LOC "$$$/Photomatix/General/Error/OutOfMemory=There was insufficient memory to convert your image"
					end
				end
				
				local formattedMsg = string.format('%s',outputMessage)
				ExportToDPXUploadTask.outputToLog(string.format('%s',outputMessage))

				if not processingSucceeded then
					LrDialogs.message("Error while exporting: " .. outputMessage, nil, "critical")
				end
			else
				-- MAC_ENV
				if retour ~= 0 then processingSucceeded = false end
			end

			LrFileUtils.delete(outputTempFile)
	end
end

